from typing import Dict
from datetime import datetime  # noqa
from wa_leg_api import waleg


def get_do_pass_by_committee(biennium: str, agency: str, committee_name: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeactionservice.asmx?op=GetDoPassByCommittee"""
    argdict = dict(biennium=biennium, agency=agency, committeeName=committee_name)
    return waleg.call("CommitteeAction", "GetDoPassByCommittee", argdict)


def get_do_pass_with_amendments_by_committee(biennium: str, agency: str, committee_name: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeactionservice.asmx?op=GetDoPassWithAmendmentsByCommittee"""
    argdict = dict(biennium=biennium, agency=agency, committeeName=committee_name)
    return waleg.call("CommitteeAction", "GetDoPassWithAmendmentsByCommittee", argdict)


def get_do_pass_with_amendments_to_sub_by_committee(biennium: str, agency: str, committee_name: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeactionservice.asmx?op=GetDoPassWithAmendmentsToSubByCommittee"""
    argdict = dict(biennium=biennium, agency=agency, committeeName=committee_name)
    return waleg.call("CommitteeAction", "GetDoPassWithAmendmentsToSubByCommittee", argdict)


def get_in_committee(biennium: str, agency: str, committee_name: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeactionservice.asmx?op=GetInCommittee"""
    argdict = dict(biennium=biennium, agency=agency, committeeName=committee_name)
    return waleg.call("CommitteeAction", "GetInCommittee", argdict)


def get_majority_report_by_committee(biennium: str, agency: str, committee_name: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeactionservice.asmx?op=GetMajorityReportByCommittee"""
    argdict = dict(biennium=biennium, agency=agency, committeeName=committee_name)
    return waleg.call("CommitteeAction", "GetMajorityReportByCommittee", argdict)


def get_minority_report_by_committee(biennium: str, agency: str, committee_name: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeactionservice.asmx?op=GetMinorityReportByCommittee"""
    argdict = dict(biennium=biennium, agency=agency, committeeName=committee_name)
    return waleg.call("CommitteeAction", "GetMinorityReportByCommittee", argdict)


def get_re_referral_by_committee(biennium: str, agency: str, committee_name: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeactionservice.asmx?op=GetReReferralByCommittee"""
    argdict = dict(biennium=biennium, agency=agency, committeeName=committee_name)
    return waleg.call("CommitteeAction", "GetReReferralByCommittee", argdict)


def get_referred_to_another_committee_by_committee(biennium: str, agency: str, committee_name: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeactionservice.asmx?op=GetReferredToAnotherCommitteeByCommittee"""
    argdict = dict(biennium=biennium, agency=agency, committeeName=committee_name)
    return waleg.call("CommitteeAction", "GetReferredToAnotherCommitteeByCommittee", argdict)


def get_referred_to_committee(biennium: str, agency: str, committee_name: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeactionservice.asmx?op=GetReferredToCommittee"""
    argdict = dict(biennium=biennium, agency=agency, committeeName=committee_name)
    return waleg.call("CommitteeAction", "GetReferredToCommittee", argdict)


def get_committee_referrals_by_committee(biennium: str, agency: str, committee_name: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeactionservice.asmx?op=GetCommitteeReferralsByCommittee"""
    argdict = dict(biennium=biennium, agency=agency, committeeName=committee_name)
    return waleg.call("CommitteeAction", "GetCommitteeReferralsByCommittee", argdict)


def get_committee_referrals_by_bill(biennium: str, bill_number: int) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeactionservice.asmx?op=GetCommitteeReferralsByBill"""
    argdict = dict(biennium=biennium, billNumber=bill_number)
    return waleg.call("CommitteeAction", "GetCommitteeReferralsByBill", argdict)


def get_removed_from_committee(biennium: str, agency: str, committee_name: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeactionservice.asmx?op=GetRemovedFromCommittee"""
    argdict = dict(biennium=biennium, agency=agency, committeeName=committee_name)
    return waleg.call("CommitteeAction", "GetRemovedFromCommittee", argdict)


def get_do_pass_substitute_by_committee(biennium: str, agency: str, committee_name: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeactionservice.asmx?op=GetDoPassSubstituteByCommittee"""
    argdict = dict(biennium=biennium, agency=agency, committeeName=committee_name)
    return waleg.call("CommitteeAction", "GetDoPassSubstituteByCommittee", argdict)


def get_without_recommendation_by_committee(biennium: str, agency: str, committee_name: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeactionservice.asmx?op=GetWithoutRecommendationByCommittee"""
    argdict = dict(biennium=biennium, agency=agency, committeeName=committee_name)
    return waleg.call("CommitteeAction", "GetWithoutRecommendationByCommittee", argdict)


def get_committee_executive_actions_by_bill(biennium: str, bill_number: int) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeactionservice.asmx?op=GetCommitteeExecutiveActionsByBill"""
    argdict = dict(biennium=biennium, billNumber=bill_number)
    return waleg.call("CommitteeAction", "GetCommitteeExecutiveActionsByBill", argdict)


def get_legislation_reported_out_of_committee(committee_name: str, agency: str, begin_date: datetime, end_date: datetime) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeactionservice.asmx?op=GetLegislationReportedOutOfCommittee"""
    argdict = dict(committeeName=committee_name, agency=agency, beginDate=begin_date, endDate=end_date)
    return waleg.call("CommitteeAction", "GetLegislationReportedOutOfCommittee", argdict)


def get_legislation_scheduled_hearings_by_committee(biennium: str, agency: str, committee_name: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeactionservice.asmx?op=GetLegislationScheduledHearingsByCommittee"""
    argdict = dict(biennium=biennium, agency=agency, committeeName=committee_name)
    return waleg.call("CommitteeAction", "GetLegislationScheduledHearingsByCommittee", argdict)



from typing import Dict
from datetime import datetime  # noqa
from wa_leg_api import waleg


def get_legislation_affecting_rcw_cite(biennium: str, rcw_cite: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/rcwciteaffectedservice.asmx?op=GetLegislationAffectingRcwCite"""
    argdict = dict(biennium=biennium, rcwCite=rcw_cite)
    return waleg.call("RcwCiteAffected", "GetLegislationAffectingRcwCite", argdict)


def get_legislation_affecting_rcw(biennium: str, rcw_cite: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/rcwciteaffectedservice.asmx?op=GetLegislationAffectingRcw"""
    argdict = dict(biennium=biennium, rcwCite=rcw_cite)
    return waleg.call("RcwCiteAffected", "GetLegislationAffectingRcw", argdict)



from typing import Dict


class WaLegApiException(Exception):
    """Throws an exception with useful info for debugging"""

    def __init__(
        self, http_error_num: int, http_error_text: str, http_text: str, args_sent: Dict
    ):
        """
        Parameters
        ----------
        http_error_num: int
            HTTP status code (e.g. 500)
        http_error_text: str
            HTTP status text (e.g. 'Internal Server Error')
        http_text: str
            Any additional text returned with the failed request
            (e.g. System.Web.Services.Protocols.SoapException)
        args_sent: Dict
            The arguments that were sent with this request
        """
        # Store relevant info in the class to be viewabile in a debugger
        self.http_error_num = http_error_num
        self.http_error_text = http_error_text
        self.http_text = http_text
        self.args_sent = args_sent.copy()

        message = f"{http_error_num} {http_error_text}\n{http_text}"
        super().__init__(message)



import requests
from typing import Dict, Any, Tuple
from bs4 import BeautifulSoup, Tag
from wa_leg_api.exceptions import WaLegApiException

WSLSITE = "http://wslwebservices.leg.wa.gov"


def unpack_array(array: Tag) -> list:
    """Parse section of return where tag == arrayofsomething
    into a list

    Parameters
    ----------
    array: bs4.Tag
        Section of returned tree to be parsed as an array

    Returns:
        list of items
    """
    answer = []

    for item in array:
        if type(item) is not Tag:
            continue
        answer.append(unpack_thing(item)[1])
    return answer


def unpack_struct(struct: Tag) -> Dict:
    """Parse a tag with children, if tag is not arrayof....

    Parameters
    ----------
    struct: bs4.Tag
        Section of returned tree to be parsed as a complex type

    Returns
    -------
    Dict
        The parsed section
    """
    answer = {}
    for item in struct:
        if type(item) is not Tag:
            continue
        name, content = unpack_thing(item)
        answer[name] = content
    return answer


def unpack_thing(thing: Tag) -> Tuple[str, Any]:
    """Parse a chunk of the returned data

    Parameters
    ----------
    thing: bs4.Tag
        root node to unpack from

    Returns
    -------
    name: str
        Name of field unpacked
    contents: Any
        - str if it's a single item
        - list if it's an array item
        - dict if it's a more complex return type
    """
    name = thing.name

    if len(thing.contents) > 1:
        if thing.name.startswith("arrayof"):
            contents = unpack_array(thing)
        else:
            contents = unpack_struct(thing)
    else:
        contents = thing.string

    return name, contents


def call(service: str, function: str, argdict: dict) -> Dict:
    """This is the backend to all the stubs

    service: str
        Which service
    function: str
        Function request inside service
    argdict: Dict
        Arguments to the request
    """
    url = f"{WSLSITE}/{service}Service.asmx/{function}"
    response = requests.get(url, params=argdict)
    if not response.ok:
        raise WaLegApiException(
            response.status_code, response.reason, response.text, argdict
        )

    body = BeautifulSoup(response.text, "html.parser")
    answer = unpack_struct(body)
    return answer


if __name__ == "__main__":
    from sponsor import get_sponsors

    result = get_sponsors("2019-20")



"""
Generates python stubs from the Washington State Legislature Schema XML Responses
"""

import requests
from bs4 import BeautifulSoup
import re
from typing import Dict, Tuple

SERVICES = [
    "Amendment",
    "CommitteeMeeting",
    "CommitteeAction",
    "Committee",
    "Legislation",
    "LegislativeDocument",
    "RcwCiteAffected",
    "SessionLaw",
    "Sponsor",
]


def snake_case(identifier: str) -> str:
    """Make a JavaScript identifier into a Python one

    Parameters
    ----------
    identifier: str
        Javascript identifier in CapitalSnakeCase or lowerSnakeCase

    Returns
    -------
    pythid: str
        Python-standard identifier in snake_case
    """
    pythid = re.sub(r"[A-Z]", lambda x: "_" + x.group(0).lower(), identifier)

    if pythid[0] == "_":
        pythid = pythid[1:]

    return pythid


def makearglists(args: Dict) -> Tuple[str, str]:
    """
    Returns the python code for argument declaration and argument passing to
    the function that does the work

    Parameters
    ----------
    args: dict
        Arg info for function returned by Wash Leg website

    Returns
    -------
    arg_declare: str
        String to paste into argument declaration of stub function
    arg_pass: str
        String to paste into backend call of stub function
    """

    for key in args:
        pytype = args[key]["type"].replace("s:", "").lower()
        if pytype == "string":
            pytype = "str"
        elif pytype == "boolean":
            pytype = "bool"
        args[key]["python_type"] = pytype
        args[key]["python_arg"] = snake_case(key)

    arg_types = [f'{args[key]["python_arg"]}: {args[key]["python_type"]}' for key in args]
    arg_declare = ", ".join(arg_types)

    all_args = ", ".join([f"{key}={args[key]['python_arg']}" for key in args])
    arg_pass = f"argdict = dict({all_args})"

    return arg_declare, arg_pass


def make_python_code(servicename: str, functionname: str, args: Dict, fp):
    """
    Generate the stub for a single service request type

    Parameters
    ----------
    servicename: str
        Which service
    functionname: str
        Which request type
    args: dict
        Argument info returned by Wash Leg XML service
    fp: file
        File to paste the stub into
    """
    arg_declare, arg_pass = makearglists(args)
    helpful_url = f'http://wslwebservices.leg.wa.gov/{servicename.lower()}service.asmx?op={functionname}'

    fp.write("\n\ndef ")
    fp.write(snake_case(functionname))
    fp.write(f"({arg_declare}) -> Dict:\n")
    fp.write(f'    """See: {helpful_url}"""\n')
    fp.write(f"    {arg_pass}\n")
    fp.write(
        f'    return waleg.call("{servicename}", "{functionname}", argdict)\n'
    )


def make_stub_files():
    """Queries the definitions of each service and at wslwebservices.leg.wa.gov
    and creates python stub files
    """
    for service in SERVICES:

        fp = open(f"{service.lower()}.py", "w")
        fp.write("from typing import Dict\n")
        fp.write("from datetime import datetime  # noqa\n")
        fp.write("from wa_leg_api import waleg\n")

        wsdl = requests.get(
            f"http://wslwebservices.leg.wa.gov/{service}Service.asmx?WSDL"
        )
        legxml = BeautifulSoup(wsdl.content, "xml")
        schema = legxml.find("s:schema")

        protocols = schema.findAll("s:element", recursive=False)
        for info in protocols:
            name = info["name"]

            if any((not name.startswith("Get"), name.endswith("Response"))):
                continue

            args = info.findAll("s:element")
            arg_dict = {arg.attrs["name"]: arg.attrs for arg in args}

            make_python_code(service, name, arg_dict, fp)
        fp.close()


if __name__ == "__main__":
    make_stub_files()



from typing import Dict
from datetime import datetime  # noqa
from wa_leg_api import waleg


def get_sponsors(biennium: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/sponsorservice.asmx?op=GetSponsors"""
    argdict = dict(biennium=biennium)
    return waleg.call("Sponsor", "GetSponsors", argdict)


def get_house_sponsors(biennium: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/sponsorservice.asmx?op=GetHouseSponsors"""
    argdict = dict(biennium=biennium)
    return waleg.call("Sponsor", "GetHouseSponsors", argdict)


def get_senate_sponsors(biennium: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/sponsorservice.asmx?op=GetSenateSponsors"""
    argdict = dict(biennium=biennium)
    return waleg.call("Sponsor", "GetSenateSponsors", argdict)


def get_requesters(biennium: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/sponsorservice.asmx?op=GetRequesters"""
    argdict = dict(biennium=biennium)
    return waleg.call("Sponsor", "GetRequesters", argdict)



from typing import Dict
from datetime import datetime  # noqa
from wa_leg_api import waleg


def get_amendments_for_year(year: int, bill_number: int) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetAmendmentsForYear"""
    argdict = dict(year=year, billNumber=bill_number)
    return waleg.call("Legislation", "GetAmendmentsForYear", argdict)


def get_amendments_for_biennium(biennium: str, bill_number: int) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetAmendmentsForBiennium"""
    argdict = dict(biennium=biennium, billNumber=bill_number)
    return waleg.call("Legislation", "GetAmendmentsForBiennium", argdict)


def get_hearings(biennium: str, bill_number: int) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetHearings"""
    argdict = dict(biennium=biennium, billNumber=bill_number)
    return waleg.call("Legislation", "GetHearings", argdict)


def get_legislation_by_request_number(biennium: str, request_number: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislationByRequestNumber"""
    argdict = dict(biennium=biennium, requestNumber=request_number)
    return waleg.call("Legislation", "GetLegislationByRequestNumber", argdict)


def get_rcw_cites_affected(biennium: str, bill_id: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetRcwCitesAffected"""
    argdict = dict(biennium=biennium, billId=bill_id)
    return waleg.call("Legislation", "GetRcwCitesAffected", argdict)


def get_session_law_chapter(biennium: str, bill_id: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetSessionLawChapter"""
    argdict = dict(biennium=biennium, billId=bill_id)
    return waleg.call("Legislation", "GetSessionLawChapter", argdict)


def get_sponsors(biennium: str, bill_id: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetSponsors"""
    argdict = dict(biennium=biennium, billId=bill_id)
    return waleg.call("Legislation", "GetSponsors", argdict)


def get_roll_calls(biennium: str, bill_number: int) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetRollCalls"""
    argdict = dict(biennium=biennium, billNumber=bill_number)
    return waleg.call("Legislation", "GetRollCalls", argdict)


def get_current_status(biennium: str, bill_number: int) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetCurrentStatus"""
    argdict = dict(biennium=biennium, billNumber=bill_number)
    return waleg.call("Legislation", "GetCurrentStatus", argdict)


def get_legislation_types() -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislationTypes"""
    argdict = dict()
    return waleg.call("Legislation", "GetLegislationTypes", argdict)


def get_total_legislation_introduced_by_date_range(begin_date: datetime, end_date: datetime, leg_type_id: int, agency_id: int, all_versions: bool) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetTotalLegislationIntroducedByDateRange"""
    argdict = dict(beginDate=begin_date, endDate=end_date, legTypeId=leg_type_id, agencyId=agency_id, allVersions=all_versions)
    return waleg.call("Legislation", "GetTotalLegislationIntroducedByDateRange", argdict)


def get_legislation(biennium: str, bill_number: int) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislation"""
    argdict = dict(biennium=biennium, billNumber=bill_number)
    return waleg.call("Legislation", "GetLegislation", argdict)


def get_legislation_introduced_since(since_date: datetime) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislationIntroducedSince"""
    argdict = dict(sinceDate=since_date)
    return waleg.call("Legislation", "GetLegislationIntroducedSince", argdict)


def get_prefiled_legislation() -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetPrefiledLegislation"""
    argdict = dict()
    return waleg.call("Legislation", "GetPrefiledLegislation", argdict)


def get_legislative_status_changes_by_bill_number(biennium: str, bill_number: int, begin_date: datetime, end_date: datetime) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislativeStatusChangesByBillNumber"""
    argdict = dict(biennium=biennium, billNumber=bill_number, beginDate=begin_date, endDate=end_date)
    return waleg.call("Legislation", "GetLegislativeStatusChangesByBillNumber", argdict)


def get_legislative_status_changes_by_bill_id(biennium: str, bill_id: str, begin_date: datetime, end_date: datetime) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislativeStatusChangesByBillId"""
    argdict = dict(biennium=biennium, billId=bill_id, beginDate=begin_date, endDate=end_date)
    return waleg.call("Legislation", "GetLegislativeStatusChangesByBillId", argdict)


def get_legislative_status_changes_by_date_range(biennium: str, begin_date: datetime, end_date: datetime) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislativeStatusChangesByDateRange"""
    argdict = dict(biennium=biennium, beginDate=begin_date, endDate=end_date)
    return waleg.call("Legislation", "GetLegislativeStatusChangesByDateRange", argdict)


def get_legislation_by_year(year: int) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislationByYear"""
    argdict = dict(year=year)
    return waleg.call("Legislation", "GetLegislationByYear", argdict)


def get_legislation_info_introduced_since(since_date: datetime) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislationInfoIntroducedSince"""
    argdict = dict(sinceDate=since_date)
    return waleg.call("Legislation", "GetLegislationInfoIntroducedSince", argdict)


def get_pre_filed_legislation_info() -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetPreFiledLegislationInfo"""
    argdict = dict()
    return waleg.call("Legislation", "GetPreFiledLegislationInfo", argdict)


def get_house_legislation_passed_house(biennium: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetHouseLegislationPassedHouse"""
    argdict = dict(biennium=biennium)
    return waleg.call("Legislation", "GetHouseLegislationPassedHouse", argdict)


def get_house_legislation_passed_senate(biennium: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetHouseLegislationPassedSenate"""
    argdict = dict(biennium=biennium)
    return waleg.call("Legislation", "GetHouseLegislationPassedSenate", argdict)


def get_senate_legislation_passed_senate(biennium: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetSenateLegislationPassedSenate"""
    argdict = dict(biennium=biennium)
    return waleg.call("Legislation", "GetSenateLegislationPassedSenate", argdict)


def get_senate_legislation_passed_house(biennium: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetSenateLegislationPassedHouse"""
    argdict = dict(biennium=biennium)
    return waleg.call("Legislation", "GetSenateLegislationPassedHouse", argdict)


def get_legislation_passed_legislature(biennium: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislationPassedLegislature"""
    argdict = dict(biennium=biennium)
    return waleg.call("Legislation", "GetLegislationPassedLegislature", argdict)


def get_legislation_passed_legislature_within_time_frame(begin_date: datetime, end_date: datetime) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislationPassedLegislatureWithinTimeFrame"""
    argdict = dict(beginDate=begin_date, endDate=end_date)
    return waleg.call("Legislation", "GetLegislationPassedLegislatureWithinTimeFrame", argdict)


def get_legislation_passed_house(biennium: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislationPassedHouse"""
    argdict = dict(biennium=biennium)
    return waleg.call("Legislation", "GetLegislationPassedHouse", argdict)


def get_legislation_passed_senate(biennium: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislationPassedSenate"""
    argdict = dict(biennium=biennium)
    return waleg.call("Legislation", "GetLegislationPassedSenate", argdict)


def get_legislation_governor_signed(biennium: str, agency: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislationGovernorSigned"""
    argdict = dict(biennium=biennium, agency=agency)
    return waleg.call("Legislation", "GetLegislationGovernorSigned", argdict)


def get_legislation_governor_veto(biennium: str, agency: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislationGovernorVeto"""
    argdict = dict(biennium=biennium, agency=agency)
    return waleg.call("Legislation", "GetLegislationGovernorVeto", argdict)


def get_legislation_governor_partial_veto(biennium: str, agency: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislationGovernorPartialVeto"""
    argdict = dict(biennium=biennium, agency=agency)
    return waleg.call("Legislation", "GetLegislationGovernorPartialVeto", argdict)


def get_published_enrolled_legislation(biennium: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetPublishedEnrolledLegislation"""
    argdict = dict(biennium=biennium)
    return waleg.call("Legislation", "GetPublishedEnrolledLegislation", argdict)


def get_legislation_passed_house_within_time_frame(begin_date: datetime, end_date: datetime) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislationPassedHouseWithinTimeFrame"""
    argdict = dict(beginDate=begin_date, endDate=end_date)
    return waleg.call("Legislation", "GetLegislationPassedHouseWithinTimeFrame", argdict)


def get_legislation_passed_senate_within_time_frame(begin_date: datetime, end_date: datetime) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislationPassedSenateWithinTimeFrame"""
    argdict = dict(beginDate=begin_date, endDate=end_date)
    return waleg.call("Legislation", "GetLegislationPassedSenateWithinTimeFrame", argdict)


def get_legislation_not_yet_introduced_in_house_of_origin(biennium: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislationNotYetIntroducedInHouseOfOrigin"""
    argdict = dict(biennium=biennium)
    return waleg.call("Legislation", "GetLegislationNotYetIntroducedInHouseOfOrigin", argdict)


def get_legislation_passed_original_body_and_not_introduced_in_opposite_body(biennium: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislationPassedOriginalBodyAndNotIntroducedInOppositeBody"""
    argdict = dict(biennium=biennium)
    return waleg.call("Legislation", "GetLegislationPassedOriginalBodyAndNotIntroducedInOppositeBody", argdict)


def get_legislative_bill_list_feature_data() -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislationservice.asmx?op=GetLegislativeBillListFeatureData"""
    argdict = dict()
    return waleg.call("Legislation", "GetLegislativeBillListFeatureData", argdict)



from typing import Dict
from datetime import datetime  # noqa
from wa_leg_api import waleg


def get_committees(biennium: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeservice.asmx?op=GetCommittees"""
    argdict = dict(biennium=biennium)
    return waleg.call("Committee", "GetCommittees", argdict)


def get_house_committees(biennium: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeservice.asmx?op=GetHouseCommittees"""
    argdict = dict(biennium=biennium)
    return waleg.call("Committee", "GetHouseCommittees", argdict)


def get_senate_committees(biennium: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeservice.asmx?op=GetSenateCommittees"""
    argdict = dict(biennium=biennium)
    return waleg.call("Committee", "GetSenateCommittees", argdict)


def get_committee_members(biennium: str, agency: str, committee_name: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeservice.asmx?op=GetCommitteeMembers"""
    argdict = dict(biennium=biennium, agency=agency, committeeName=committee_name)
    return waleg.call("Committee", "GetCommitteeMembers", argdict)


def get_active_committees() -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeservice.asmx?op=GetActiveCommittees"""
    argdict = dict()
    return waleg.call("Committee", "GetActiveCommittees", argdict)


def get_active_house_committees() -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeservice.asmx?op=GetActiveHouseCommittees"""
    argdict = dict()
    return waleg.call("Committee", "GetActiveHouseCommittees", argdict)


def get_active_senate_committees() -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeservice.asmx?op=GetActiveSenateCommittees"""
    argdict = dict()
    return waleg.call("Committee", "GetActiveSenateCommittees", argdict)


def get_active_committee_members(agency: str, committee_name: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeeservice.asmx?op=GetActiveCommitteeMembers"""
    argdict = dict(agency=agency, committeeName=committee_name)
    return waleg.call("Committee", "GetActiveCommitteeMembers", argdict)



from typing import Dict
from datetime import datetime  # noqa
from wa_leg_api import waleg


def get_committee_meetings(begin_date: datetime, end_date: datetime) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeemeetingservice.asmx?op=GetCommitteeMeetings"""
    argdict = dict(beginDate=begin_date, endDate=end_date)
    return waleg.call("CommitteeMeeting", "GetCommitteeMeetings", argdict)


def get_revised_committee_meetings(changed_since_date: datetime) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeemeetingservice.asmx?op=GetRevisedCommitteeMeetings"""
    argdict = dict(changedSinceDate=changed_since_date)
    return waleg.call("CommitteeMeeting", "GetRevisedCommitteeMeetings", argdict)


def get_committee_meeting_items(agenda_id: int) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/committeemeetingservice.asmx?op=GetCommitteeMeetingItems"""
    argdict = dict(agendaId=agenda_id)
    return waleg.call("CommitteeMeeting", "GetCommitteeMeetingItems", argdict)



from typing import Dict
from datetime import datetime  # noqa
from wa_leg_api import waleg


def get_amendments(year: int) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/amendmentservice.asmx?op=GetAmendments"""
    argdict = dict(year=year)
    return waleg.call("Amendment", "GetAmendments", argdict)



# Always prefer setuptools over distutils
from setuptools import setup
import pathlib

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name="wa_leg_api",  # Required
    version="0.0.0",  # Required
    description="A python API for the Washington State Legislature web services",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/j-carson/wa-leg-api",
    # Classifiers help users find your project by categorizing it.
    #
    # For a list of valid classifiers, see https://pypi.org/classifiers/
    classifiers=[  # Optional
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 2 - Pre-Alpha",
        # Indicate who your project is intended for
        "Intended Audience :: Developers",
        # Pick your license as you wish
        "License :: OSI Approved :: MIT License",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate you support Python 3. These classifiers are *not*
        # checked by 'pip install'. See instead 'python_requires' below.
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent"
    ],
    keywords=["washington-state",
              "legislature",
              "government-data"],  # Optional
    # When your source code is in a subdirectory under the project root, e.g.
    # `src/`, it is necessary to specify the `package_dir` argument.
    packages=["wa_leg_api"],
    python_requires=">=3.7, <4",
    install_requires=[
        "beautifulsoup4",
        "requests",
    ],  # Optional
    extras_require={  # Optional
        "dev": ["lxml", "pytest"],
        "docs": ["sphinx", "recommonmark"]
    },
    # List additional URLs that are relevant to your project as a dict.
    project_urls={"Source": "https://github.com/j-carson/wa-leg-api",
                 "Documentation": "https://wa-leg-api.readthedocs.io"},  
)



from typing import Dict
from datetime import datetime  # noqa
from wa_leg_api import waleg


def get_session_law_by_bill(biennium: str, bill_number: int) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/sessionlawservice.asmx?op=GetSessionLawByBill"""
    argdict = dict(biennium=biennium, billNumber=bill_number)
    return waleg.call("SessionLaw", "GetSessionLawByBill", argdict)


def get_bill_by_chapter_number(year: int, session: int, chapter_number: int) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/sessionlawservice.asmx?op=GetBillByChapterNumber"""
    argdict = dict(year=year, session=session, chapterNumber=chapter_number)
    return waleg.call("SessionLaw", "GetBillByChapterNumber", argdict)


def get_chapter_numbers_by_year(year: int) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/sessionlawservice.asmx?op=GetChapterNumbersByYear"""
    argdict = dict(year=year)
    return waleg.call("SessionLaw", "GetChapterNumbersByYear", argdict)


def get_session_law_by_bill_id(biennium: str, bill_id: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/sessionlawservice.asmx?op=GetSessionLawByBillId"""
    argdict = dict(biennium=biennium, billId=bill_id)
    return waleg.call("SessionLaw", "GetSessionLawByBillId", argdict)


def get_session_law_by_initiative_number(initiative_number: int) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/sessionlawservice.asmx?op=GetSessionLawByInitiativeNumber"""
    argdict = dict(initiativeNumber=initiative_number)
    return waleg.call("SessionLaw", "GetSessionLawByInitiativeNumber", argdict)



from typing import Dict
from datetime import datetime  # noqa
from wa_leg_api import waleg


def get_documents_by_class(biennium: str, document_class: str, named_like: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislativedocumentservice.asmx?op=GetDocumentsByClass"""
    argdict = dict(biennium=biennium, documentClass=document_class, namedLike=named_like)
    return waleg.call("LegislativeDocument", "GetDocumentsByClass", argdict)


def get_documents(biennium: str, named_like: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislativedocumentservice.asmx?op=GetDocuments"""
    argdict = dict(biennium=biennium, namedLike=named_like)
    return waleg.call("LegislativeDocument", "GetDocuments", argdict)


def get_document_classes(biennium: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislativedocumentservice.asmx?op=GetDocumentClasses"""
    argdict = dict(biennium=biennium)
    return waleg.call("LegislativeDocument", "GetDocumentClasses", argdict)


def get_all_documents_by_class(biennium: str, document_class: str) -> Dict:
    """See: http://wslwebservices.leg.wa.gov/legislativedocumentservice.asmx?op=GetAllDocumentsByClass"""
    argdict = dict(biennium=biennium, documentClass=document_class)
    return waleg.call("LegislativeDocument", "GetAllDocumentsByClass", argdict)
